﻿
#region

using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;

#endregion

namespace WpfDockingAndRibbonTest.Controls
{
   public class BuilderControl : UserControl, IDisposable
    {
        #region Private Fields

        private bool m_isFirstLoaded;
        private string m_caption;
        protected readonly RibbonMerge RibbonMerge;
        #endregion

        #region Properties
        public Guid UniqueId { get; set; }
        /// <summary>
        /// Caption shown on Hosting Tab
        /// </summary>
        public string Caption
        {
            get { return m_caption; }
            set
            {
                if (m_caption != value)
                {
                    //FireOnBuilderControlCaptionChanged(new EventArgs<string>(value));
                    m_caption = value;
                }
            }
        }
        
        #endregion

        #region Ctor

       public BuilderControl()
       {
           UniqueId = Guid.NewGuid();
           CustomName = UniqueId.ToString();
           Dispatcher.UnhandledException += Dispatcher_UnhandledException;
       }

       public BuilderControl(string customName)
       {
           UniqueId = Guid.NewGuid();
           CustomName = customName;
           Dispatcher.UnhandledException += Dispatcher_UnhandledException;
           Loaded += OnLoaded;
       }


        public BuilderControl(string customName, Window parentWindow, RibbonMerge ribbonMerge)
        {
            UniqueId = Guid.NewGuid();
            CustomName = customName;
            RibbonMerge = ribbonMerge;
            ParentWindow = parentWindow;
            Dispatcher.UnhandledException += Dispatcher_UnhandledException;
        }
        #endregion

        #region DependencyProperty ProjectState
        public string CustomName { get; }

       public Window ParentWindow { get; }

       public bool IsSaved
       {
           get { return (bool)GetValue(IsSavedProperty); }
           set { SetValue(IsSavedProperty, value); }
       }

       public static readonly DependencyProperty IsSavedProperty =
            DependencyProperty.Register("IsSaved", typeof (bool), typeof (BuilderControl), new PropertyMetadata(true, OnIsSavedChanged));

        private static void OnIsSavedChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            var sender = (BuilderControl)obj;

            var e = new RoutedPropertyChangedEventArgs<bool>((bool)args.OldValue, (bool)args.NewValue, IsSavedChangedEvent);
            sender.OnIsSavedChanged(e);

        }
        #region OnTsProjectStateStateChanged
        protected virtual void OnIsSavedChanged(RoutedPropertyChangedEventArgs<bool> args)
        {
            RaiseEvent(args);
        }
        /// <summary>
        /// The is saved changed event
        /// </summary>
        public static readonly RoutedEvent IsSavedChangedEvent = EventManager.RegisterRoutedEvent(
            "IsSavedChanged", RoutingStrategy.Bubble,
            typeof(RoutedPropertyChangedEventHandler<bool>), typeof(BuilderControl));

        /// <summary>
        /// Occurs when [is saved changed].
        /// </summary>
        public event RoutedPropertyChangedEventHandler<bool> IsSavedChanged
        {
            add { AddHandler(IsSavedChangedEvent, value); }
            remove { RemoveHandler(IsSavedChangedEvent, value); }
        }
        #endregion
        #endregion

        void Dispatcher_UnhandledException(object sender, DispatcherUnhandledExceptionEventArgs e)
        {
            //var trainsIdeException = new TrainsIdeException(Errors.CmnError0003(Errors.Str_ContactTecnicalSupport), e.Exception);
            //FireOnBuilderControlException(new EventArgs<TrainsIdeException>(trainsIdeException));
        }

        //#region Open ProjectItem Action Request event
        //public event EventHandler<OpenProjectItemActionRequestEventArgs> OpenProjectItemActionRequest;
        //public void FireOpenProjectItemActionRequest(OpenProjectItemActionRequestEventArgs e)
        //{
        //    var handler = OpenProjectItemActionRequest;
        //    handler?.Invoke(this, e);
        //}
        //#endregion


        //#region Add New ProjectItem Action Request event
        //public event EventHandler<AddNewProjectItemActionRequestEventArgs> AddNewProjectItemActionRequest;
        //public void FireAddNewProjectItemActionRequest(AddNewProjectItemActionRequestEventArgs e)
        //{
        //    var handler = AddNewProjectItemActionRequest;
        //    handler?.Invoke(this, e);
        //}
        //#endregion
        //#region Exception event
        //private event EventHandler<EventArgs<TrainsIdeException>> OnBuilderControlException;

        //public event EventHandler<EventArgs<TrainsIdeException>> OnBuilderControlExceptionHandler
        //{
        //    add { OnBuilderControlException += value; }
        //    remove { OnBuilderControlException -= value; }
        //}

        //public virtual void FireOnBuilderControlException(EventArgs<TrainsIdeException> e)
        //{
        //    var handler = OnBuilderControlException;
        //    handler?.Invoke(this, e);
        //}
        //#endregion

        //#region Caption changed event
        //private event EventHandler<EventArgs<string>> OnBuilderControlCaptionChanged;

        //public event EventHandler<EventArgs<string>> OnBuilderControlCaptionChangedHandler
        //{
        //    add { OnBuilderControlCaptionChanged += value; }
        //    remove { OnBuilderControlCaptionChanged -= value; }
        //}

        //public void FireOnBuilderControlCaptionChanged(EventArgs<string> e)
        //{
        //    var handler = OnBuilderControlCaptionChanged;
        //    handler?.Invoke(this, e);
        //}
        //#endregion

        //#region Request Busy changed event
        //private event EventHandler<EventArgs<string>> OnBuilderControlBusyChanged;

        //public event EventHandler<EventArgs<string>> OnBuilderControlBusyChangedHandler
        //{
        //    add { OnBuilderControlBusyChanged += value; }
        //    remove { OnBuilderControlBusyChanged -= value; }
        //}

        //public void FireOnBuilderControlBusyChanged(EventArgs<string> e)
        //{
        //    var handler = OnBuilderControlBusyChanged;
        //    handler?.Invoke(this, e);
        //}
        //#endregion

        //#region Control First Load event
        //private event EventHandler<EventArgs<string>> OnBuilderControlLoaded;

        //public event EventHandler<EventArgs<string>> OnBuilderControlLoadedHandler
        //{
        //    add { OnBuilderControlBusyChanged += value; }
        //    remove { OnBuilderControlBusyChanged -= value; }
        //}

        //public void FireOnBuilderControlLoaded(EventArgs<string> e)
        //{
        //    var handler = OnBuilderControlLoaded;
        //    handler?.Invoke(this, e);
        //}
        //#endregion

        //#region Virtual Methods
        //public virtual void Dispose()
        //{
        //}

        //public virtual void DoCloseActions()
        //{
        //}

        //public virtual void DoSaveActions()
        //{}

        //public virtual void DoPostSaveActions()
        //{ }

        //#endregion

        #region UserControl Events
        private void OnLoaded(object sender, RoutedEventArgs routedEventArgs)
        {
            //if (!m_isFirstLoaded)
            //{
            //    m_isFirstLoaded = true;
            //    FireOnBuilderControlBusyChanged(new EventArgs<string>(string.Empty));
            //}
        }

        #endregion
        public void Dispose()
       {
           
       }
    }
}
