﻿
#region

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Media;

using Telerik.Windows.Controls;
using Telerik.Windows.Controls.Docking;

#endregion

namespace WpfDockingAndRibbonTest.Controls
{
    public class RibbonMerge
    {
        #region Private Fields

        private ImageSource m_hostImageSource;

        #endregion

        #region Properties
        public RadDocking MainDockManager
        { get; set; }

        public System.Windows.Controls.Ribbon.Ribbon MainRibbon
        { get; set; }

        #endregion

        #region Ctor
        public RibbonMerge(RadDocking mainDockManager, System.Windows.Controls.Ribbon.Ribbon mainRibbon)
        {
            MainDockManager = mainDockManager;
            MainRibbon = mainRibbon;
        }
        #endregion
        
        #region Public Methods
        /// <summary>
        /// Loads the user control inside docking manager, merging user control ribbon with the main window ribbon.
        /// </summary>
        /// <param name="userControl">The user control.</param>
        /// <param name="caption">The caption.</param>
        /// <example>   var userControl = new ImportControl(); 
        ///     this.LoadUserControl(userControl, "Import!");  </example>
        public void LoadUserControl(BuilderControl userControl, string caption = "")
        {
            // hooks the Loaded/unloaded events to manage the dynamic ribbon merging
            userControl.Loaded += UserControl_Loaded;
            userControl.Unloaded += UserControl_Unloaded;

            DocumentHost documentHost = null;
            if (MainDockManager.DocumentHost == null)
            {
                documentHost = new DocumentHost();
                MainDockManager.DocumentHost = documentHost;
            }
            else
            {
                documentHost = (DocumentHost) MainDockManager.DocumentHost;
            }
            var hostContainer = new RadSplitContainer();
            var hostGroup = new RadPaneGroup();

            var radDocumentPane = new RadDocumentPane { Header = userControl.Name, Content = userControl };
            radDocumentPane.Loaded += RadDocumentPaneOnLoaded;
            radDocumentPane.Unloaded += RadDocumentPaneOnUnloaded;
            hostGroup.AddItem(radDocumentPane, DockPosition.Center );

            hostContainer.Items.Add(hostGroup);
            documentHost.AddSplitItem(hostContainer);


            // adds uc to docking manager
            //DockingManager.SetHeader(userControl, caption);
            //DockingManager.SetSideInDockedMode(userControl, DockSide.Tabbed);
            //DockingManager.SetState(userControl, DockState.Document);

            //DockingManager.SetCanDockonFloat(userControl, false);
            //DockingManager.SetCanFloat(userControl, false);
            //DockingManager.SetCanDock(userControl, false);

            //MainDockManager.Children.Add(userControl);
        }

        private void RadDocumentPaneOnUnloaded(object sender, RoutedEventArgs routedEventArgs)
        {
        }

        private void RadDocumentPaneOnLoaded(object sender, RoutedEventArgs routedEventArgs)
        {
        }

        public void SetUserControlCaption(BuilderControl builderControl, string caption)
        {
            //if (builderControl != null && MainDockManager != null && MainDockManager.Children.Contains(builderControl))
            //{
            //    DockingManager.SetHeader(builderControl, caption);
            //}
        }
        #endregion

        #region Private Methods
        /// <summary>
        /// Handles the Loaded event of the UserControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            var t = sender as BuilderControl;
            if (t != null)
            {
                var childrenList = new List<System.Windows.Controls.Ribbon.Ribbon>();
                // gets ribbon elements from user control
                t.GetChildOfType(ref childrenList);
                if (childrenList.Any())
                {
                    // consider the 1st ribbon only
                    var r = childrenList.First();
                    var moveList = r.Items.OfType<System.Windows.Controls.Ribbon.RibbonTab>().ToList();
                    // move ribbon from usercontrol to main window, tagging them with usercontrol unique id (to be able to remove them later)
                    MoveRibbonTabs(r, MainRibbon, moveList, true, t.UniqueId);
                    r.Visibility = Visibility.Collapsed;
                }
            }
        }

        /// <summary>
        /// Handles the Unloaded event of the UserControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void UserControl_Unloaded(object sender, RoutedEventArgs e)
        {
            var t = sender as BuilderControl;
            var r = t?.GetChildOfType<System.Windows.Controls.Ribbon.Ribbon>();
            if (r != null)
            {
                // get ribbon elements to remove from main windows
                var moveList = MainRibbon.Items.OfType<System.Windows.Controls.Ribbon.RibbonTab>().Where(tab => tab.Tag is Guid && ((Guid)tab.Tag) == t.UniqueId).ToList();
                MoveRibbonTabs(MainRibbon, r, moveList, false);
                //(this.MainRibbon.Items[0] as System.Windows.Controls.Ribbon.RibbonTab).IsSelected = true;
            }
        }
        /// <summary>
        /// Moves the ribbon tabs.
        /// </summary>
        /// <param name="ribbonSource">The ribbon source.</param>
        /// <param name="ribbonDestination">The ribbon destination.</param>
        /// <param name="ribbonTabList">The ribbon tab list.</param>
        /// <param name="loading">if set to <c>true</c> [loading].</param>
        /// <param name="tagValue">The tag value.</param>
        private void MoveRibbonTabs(System.Windows.Controls.Ribbon.Ribbon ribbonSource, System.Windows.Controls.Ribbon.Ribbon ribbonDestination, List<System.Windows.Controls.Ribbon.RibbonTab> ribbonTabList, bool loading, object tagValue = null)
        {
            ribbonTabList.ForEach(t =>
            {
                t.Tag = tagValue;
                ribbonSource.Items.Remove(t);
                ribbonDestination.Items.Add(t);
            });
            if (loading)
            {
                if (ribbonDestination.ApplicationMenu != null && ribbonSource.ApplicationMenu != null)
                {
                    m_hostImageSource = ribbonDestination.ApplicationMenu.SmallImageSource;
                    ribbonDestination.ApplicationMenu.SmallImageSource = ribbonSource.ApplicationMenu.SmallImageSource;
                }
            }
            else
            {
                if (ribbonSource.ApplicationMenu != null)
                {
                    ribbonSource.ApplicationMenu.SmallImageSource = m_hostImageSource;
                    m_hostImageSource = null;
                }
            }
            if (loading && ribbonTabList.Any())
            {
                var ribbonTab = ribbonDestination.Items.OfType<System.Windows.Controls.Ribbon.RibbonTab>().FirstOrDefault(rt => string.CompareOrdinal(rt.Header.ToString(), ribbonTabList[0].Header.ToString()) == 0);
                if (ribbonTab != null)
                    ribbonTab.IsSelected = true;
            }
        }
        #endregion
    }
}
